#
# pushare - P2P file sharing system
#
# (c) Fuktommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: $
#

package Pushare::Gateway;
use Pushare::Cache;
use Pushare::Config;

$cachedir = $Pushare::Config::cachedir;

sub escape($) {
	local($_) = @_;
	s/&/&amp;/g;
	s/</&lt;/g;
	s/>/&gt;/g;
	s/'/&acute;/g;
	s/\r//g;
	s/\n/<br>/g;
	return $_;
}

sub printHeader($) {
	my($title) = @_;
	main::printHeader("200 OK");
	print "Content-Type: text/html; charset=UTF-8\r\n",
	      "Content-Language: ja\r\n",
	      "\r\n";
	print '<?xml version="1.0" encoding="UTF-8"?>',
	      '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"',
	      ' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
	      '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">',
	      '<head>',
	      '<meta http-equiv="Content-Language" content="ja" />',
	      '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />',
	      "<meta http-equiv='Content-Style-Type' content='text/css' />",
	      "<title>$title</title>",
	      "<link rev='made' href='mailto:fuktommy\@users.sourceforge.net' />",
	      "<link rel='contents' href='/gw/' />",
	      "<style type='text/css'>",
	      " ul { list-style: none; }",
	      " h1 { color: red; }",
	      " em { font-size: x-large }",
	      " p  { margin: 0px; padding: 0px }",
	      " .name { color: green; font-weight: bold; }",
	      "</style></head>";
	print "<body><p>",
	      "<a href='/gw/cache'>CACHE</a> | ",
	      "<a href='/gw/search'>SEARCH</a>",
	      "</p>";

}

sub print302($) {
	my($next) = @_;
	main::printHeader("200 OK");
	print "Content-Type: text/html; charset=UTF-8\r\n",
	      "Content-Language: ja\r\n",
	      "\r\n";
	print '<?xml version="1.0" encoding="UTF-8"?>',
	      '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"',
	      ' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
	      '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">',
	      '<head>',
	      '<meta http-equiv="Content-Language" content="ja" />',
	      '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />',
	      '<meta http-equiv="Content-Script-Type" content="text/javascript" />',
	      "<title>moved</title>",
	      "<link rev='made' href='mailto:fuktommy\@users.sourceforge.net' />",
	      "<link rel='contents' href='/gw/' />",
	      "</head>";
	print "<body><p>Click and jump to <a href='$next'>$next</a></p>\n",
	      "<script type='text/javascript'>window.location.href = '$next';</script>",
	      "</body></html>\n";
}

sub printCache() {
	printHeader("cache - pushare");
	print "<h1>cache - pushare</h1><ul>\n";
	local $_;
	my %name = Pushare::Cache::list();
	foreach (glob "$cachedir/*.dat") {
		my $id = $_;
		$id =~ s|$cachedir/||;  $id =~ s|\.dat$||;
		my $name = $name{$id};
		$name = "(null)" unless ($name);
		$name = escape($name);
		my $size = (-s $_);
		$size = int($size/100000)/10;
		my $xname = main::encode($name);
		print "<li>",
		      " <a href='/gw/ccmd?id=$id&amp;dist=1'>DIST</a>",
		      " <a href='/gw/ccmd?id=$id&amp;delete=1'>DELETE</a>",
		      " <a href='/gw/ccmd?id=$id&amp;convert=1'>CONVERT</a>",
		      " - <a href='/gw/get/$id/$xname'>$name</a>($size MB)",
		      "</li>\n";

	}
	print "</ul>",
	      "<p><a href='/gw/ccmd?update=1'>UPDATE</a></p>",
	      "</body></html>\n";
}

sub printSearch() {
	printHeader("search - pushare");
	print "<h1>search - pushare</h1><ul>\n";
	my @search = ();
	if (open IN, $Pushare::Config::searchlist) {
		@search = <IN>;
		chomp @search;
		close IN;
	}
	local $_;
	foreach (@search) {
		my $x = escape($_);
		my $y = main::encode($_);
		print "<li>",
		      "<a href='/gw/scmd?delete=1&amp;exp=$y'>DELETE</a>",
		      " - $x",
		      "</li>\n";
	}
	print "</ul>",
	      "<form method='get' action='/gw/scmd'><p>",
	      "<input name='exp' size='30' tabindex='4' accesskey='e' value='' />",
	      "<input type='submit' value='ADD' name='add' tabindex='5' accesskey='a' />",
	      "</p></form>",
	      '<p>Ex: $arg{name}=~/jpg/i &amp;&amp; $arg{size}&lt;10*1024*1024</p>',
	      "</body></html>\n";
}

sub getFile($$) {
	my($id, $name) = @_;
	my $suffix = "";
	if ($name =~ /\.([^.]+)$/) {
		$suffix = $1;
	}
	my $type = "application/octet-stream";
	if ($Pushare::Config::mimeType{$suffix}) {
		$type = $Pushare::Config::mimeType{$suffix};
	}
	unless (open IN, "$cachedir/$id.dat") {
		main::print404();
		return;
	}
	main::printHeader("200 OK");
	print "Content-Type: $type\r\n",
	      "Content-Lenget: ", (-s "$cachedir/$id.dat"), "\r\n",
	      "\r\n";
	local $_;
	while (<IN>) {
		print;
	}
	close IN;
}

sub command_cache() {
	my %arg = main::args($query_string);
	if ($arg{delete} && $arg{id}) {
		Pushare::Cache::delete($arg{id});
		print302("/gw/cache");
	} elsif ($arg{convert} && $arg{id}) {
		Pushare::Cache::convert($arg{id});
		print302("/gw/cache");
	} elsif ($arg{dist} && $arg{id}) {
		Pushare::Util::addDist(time, $arg{id});
		my $myself = Pushare::NodeList::myself();
		$myself = $$myself{host};
		my $name = Pushare::Cache::name($arg{id});
		$name = main::encode($name);
		Pushare::NodeList::relayDist("host=$myself&port=$port&id=$arg{id}&name=$name");
		print302("/gw/cache");
	} elsif ($arg{update}) {
		Pushare::Cache::update();
		print302("/gw/cache");
	} else {
		main::print404;
	}
}

sub command_search() {
	my %arg = main::args($query_string);
	if ($arg{delete} && $arg{exp}) {
		Pushare::Search::delete($arg{exp});
		print302("/gw/search");
	} elsif ($arg{add} && $arg{exp}) {
		Pushare::Search::add($arg{exp});
		print302("/gw/search");
	} else {
		main::print404;
	}
}

#------------------------------------------------------------------#
sub main() {
	$path = $main::path;
	$port = $main::port;
	$query_string = $main::query_string;

	if ($main::remote_addr !~ /$Pushare::Config::adminAddr/) {
		main::print404();

	} elsif (($path eq "/") || ($path eq "/gw") ||
		 ($path eq "/gw/") || ($path eq "/gw/cache")) {
		printCache();

	} elsif ($path eq "/gw/search") {
		printSearch();

	} elsif ($path =~ m|^/gw/get/([0-9a-f]{32})/([0-9A-Za-z.%]+)$|) {
		my($id, $name) = ($1, $2);
		getFile($id, $name);

	} elsif ($path eq "/gw/ccmd") {
		command_cache();

	} elsif ($path eq "/gw/scmd") {
		command_search();

	} else {
		main::print404();

	}
}

1;
