#
# pushare - P2P file sharing system
#
# (c) Fuktommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: $
#

package Pushare::Node;
use IPC::Open2;
use Socket;
use Pushare::Config;

$cachedir = $Pushare::Config::cachedir;

#
# constructor
# args: host, port
#
sub new($$$) {
	my($this, $host, $port) = @_;
	$this = { host => $host,
		  port => $port,
		};
	return bless $this;
}

#
# connect other node
# args: path, options
# options: file
#
sub talk($$@) {
	my($this, $path, %opt) = @_;
	my($host, $port) = ($$this{host}, $$this{port});
	local $_;
	my $timeout = $Pushare::Config::timeout;
	my $bufsize = $Pushare::Config::bufsize;
	#warn "$host:$port$path";
	my @buf = ();
	pipe IN, OUT;
	if (fork) {
		close OUT;
		local $_;
		local $SIG{CHLD} = "DEFAULT";
		my $entity = 0;
		while (<IN>) {
			if ((! $entity) && (/^[\r\n]*$/)) {
				$entity = 1;
				if ($opt{file}) {
					open OUT2, "> $cachedir/$opt{file}.dat" or die;
				}
			} elsif (! $entity) {
				next;
			} elsif ($opt{file}) {
				print OUT2;
			} else {
				push @buf, $_;
			}
		}
		close IN;
		wait;
		if ($opt{file}) {
			close OUT2;
			if (-s "$cachedir/$opt{file}.dat") {
				my($IN, $OUT);
				my $pid = open2 $IN, $OUT, $Pushare::Config::command{md5sum}, "$cachedir/$opt{file}.dat";
				close $OUT;
				my $md5 = <$IN>;
				close $IN;
				wait;
				$md5 = substr $md5, 0, 32;
				if ($md5 eq $opt{file}) {
					return (-s "$cachedir/$opt{file}.dat");
				}
			}
			if (-e "$cachedir/$opt{file}.dat") {
				unlink "$cachedir/$opt{file}.dat";
			}
			return 0;
		} else {
			s/[\r\n]//g foreach @buf;
			return @buf;
		}

	} else {
		close IN;
		my($addr, $sockaddr);
		my $try = $Pushare::Config::retry;
		local $SIG{ALRM} = sub { close OUT;
					 die "$host:$port: connection timeout."; };
		START:
		alarm $timeout;
		unless ($addr = inet_aton $host) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host: faild to resolv. $!";
		}
		$sockaddr = pack_sockaddr_in($port, $addr);
		unless (socket SOCKET, PF_INET, SOCK_STREAM, 0) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "failed to open socket. $!";
		}
		unless (connect SOCKET, $sockaddr) {
			if ($try-- > 0) {
				sleep 3;
				goto START;
			}
			close OUT;
			die "$host:$port: fault to connect. $!";
		}
		select SOCKET; $| = 1;
		local $, = "\r\n";
		local $_;
		alarm $timeout;
		my $ver = $Pushare::Config::version;
		print "GET $path HTTP/1.0",
		      "User-Agent: pushare/$ver",
		      "Host: $host:$port",
		      "Accept: text/plain",
		      "Connection: close",
		      "\r\n";
		my $buf;
		while (read SOCKET, $buf, $bufsize) {
			print OUT $buf;
			alarm $timeout;
		}
		alarm 0;
		close SOCKET;
		close OUT;
		exit;
	}
}

#
# to string
#
sub toString($) {
	my($this) = @_;
	my($host, $port) = ($$this{host}, $$this{port});
	return "$host:$port";
}

#
# ping node
#
sub ping($) {
	my($this) = @_;
	my @buf = $this->talk("/ping");
	return ((defined $buf[0])&&($buf[0] eq "PONG"));
}

#
# connect
#
sub join($) {
	my($this) = @_;
	my $port = $main::port;
	my @buf = $this->talk("/join?port=$port");
	return @buf;
}

#
# unconnect
#
sub bye($) {
	my($this) = @_;
	my $port = $main::port;
	my @buf = $this->talk("/bye?port=$port");
	return (@buf > 0) && ($buf[0] eq "BYEBYE");
}

1;
